/********************************************************
Copyright (C) 2011, Kengo NAGASHIMA.

This program is free software: you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation, either
version 3 of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE.  See the GNU General Public License
for more details.

You should have received a copy of the GNU General Public
License along with this program.
If not, see <http://www.gnu.org/licenses/>
********************************************************/

/********************************************************
  Macro name: %kmdata()
  Author: Kengo NAGASHIMA
  Version: 2.2.0
  Last Updated Date: 06/23/2011
  System Recommends:
      Windows XP 32 bit, SAS 9.2,
      set up SAS output fint size = 12pt
  System Requirements:
      Windows XP / Vista / 7, SAS 9.1.3 or later,
      Base SAS, SAS/STAT, SAS/GRAPH
********************************************************/

/********************************************************
  Make graph datasets for extended Kaplan-Meier plots
    data: a input dataset name
    time: a survival time variable name
    stra: a stratum variable name
    censor: a censor variable name
    censorv: censored variable code
    out: output graph dataset name
    anno: output annotate dataset name
    afont: no. at risk text font
    failure: 0 = a plot of estimated survival function
             1 = a plot of estimated 1 - survival function
    adjbase: 0 = none
             1 = baseline adjustment with covariates
                 see the help PHREG procedure baseline statment
    adjbase_data: a dataset for baseline adjustment
    covariates: covariates for adjustment
    ci: 0 = none
        1 = show confidence interval
    pattern: confidence interval filling pattern
        see [Annotate Dictionary STYLE Variable]
    censEXT: 0 = none
             1 = censor extension (needle)
    cHeight: extended censor height
    cWidth: extended censor width
    size: font size of at risk, test and hazard ratio
    step: at risk, MST & HR interval step size
    atrisk: 0 = none
            1 = no. at risk
    atriskorder: no. at risk order
    base: at risk base position
    label: at risk label
    test: 0 = none
          1 = show homogeneity test
    testX: = x position (% of data area)
    testY: = y position (% of data area)
    type: logrank = log-rank test
          wilcoxon = generalized wilxocon test
          likelihoodratio = likelihood ratio test
    hr: 0 = none
            1 = show hazard ratio
    hrX: x position (% of data area)
    hrY: y position (% of data area)
    MST: 0 = none
         1 = show median survival time
    MlabX: x position of strata label (% of data area)
    MmedX: x position of median (% of data area)
    MciX: x position of confidence interval (% of data area)
    MSTY: y position (% of data area)
********************************************************/
%macro km_data(
  data, time, stra, censor, censorv,
  out = graph, anno = anno,
  afont = "'Times New Roman'",

  failure = 0,

  adjbase = 0, adjbase_data = , covariates = ,

  CI = 0, Pattern = x4,

  censEXT = 0, cHeight = 0.02, cWidth = 5,

  Size = 3, Step = 5, 

  atrisk = 0, atriskorder = 0 to 0,
  Base = 0, Label = "No. at risk",

  Test = 0, TestX = 90, TestY = 100, Type = logrank,
  HR = 0, HRX = 90, HRY = 95,
  MST = 0, MlabX = 60, MmedX = 80, MciX = 98, MSTY = 95
);

%if "&adjbase." = "1" %then %do;
  %let censEXT = 0;
  %let atrisk = 0;
  %let Test = 0;
  %let HR = 0;
  %let MST = 0;
%end;

%if "&stra." = "" %then %do;
%let stra = dstra;
data &data.;
  set &data.; dstra = 1;
%end;

ods results off; ods exclude all;
proc lifetest data = &data. outsurv = &out.;
  time &time. * &censor.(&censorv.);
  strata &stra.;
  ods output
    HomTests = Test CensoredSummary = CS
    ProductLimitEstimates = PLE Quartiles = QEST;
run;
%if "&sysver." = "9.2" %then proc phreg;
%else %if "&sysver." = "9.1" %then proc tphreg;
data = &data.;
  class &stra. / param = ref ref = first;
  model &time. * &censor.(&censorv.) = &stra. / rl ties=exact;
  ods output ParameterEstimates = HazardRatio;
run;
%if "&adjbase." = "1" %then %do;
  %if "&sysver." = "9.2" %then proc phreg;
  %else %if "&sysver." = "9.1" %then proc tphreg;
  data = &data.;
    class &stra. / param = ref ref = first;
    model &time. * &censor.(&censorv.) = &stra. &covariates. / ties=exact;
    baseline out = &out. covariates = &adjbase_data. survival = _ALL_ / nomean;
  run;
  ods select all; ods results on;
  proc sort data = &out.; by &stra.;
  data &out.;
    set &out.; by &stra.;
    retain STRATUM 1;
    if last.&stra. then STRATUM = STRATUM + 1;
    _CENSOR_ = 0;
    rename LowerSurvival = SDF_LCL UpperSurvival = SDF_UCL;
  data xx&out.;
  set &out.;
%end;
ods select all; ods results on;

proc sort data = &out.; by &stra.;
data &out.;
  set &out.; retain CSDF 0;
  %if "&failure." = "1" %then %do;
    SURVIVAL = 1 - SURVIVAL;
    SDF_LCL = 1 - SDF_LCL;
    SDF_UCL = 1 - SDF_UCL;
  %end;
  if Survival = . then Survival = CSDF;
  else CSDF = Survival;
  drop &stra.;
data &out.;
  set
    &out.(where=(_CENSOR_ ^= 1) drop=CSDF)
    &out.(where=(_CENSOR_ = 1));
  drop _CENSOR_;
proc sort data = &out.; by &time.;
data _null_;
  set CS(where=(control_var^="-"));
  call symput("StraN", cat(_N_));
data &out.;
  merge
  %do i = 1 %to &StraN.;
    &out.(where=(STRATUM=&i.)
      rename=(Survival=SV&i. SDF_LCL=SL&i. SDF_UCL=SU&i.)
                           drop=CSDF)
    &out.(where=(STRATUM=&i. & CSDF&i.^=.)
      rename=(CSDF=CSDF&i. SDF_LCL=SL&i. SDF_UCL=SU&i.)
                           drop=Survival)
  %end; ;
  by &time.; drop STRATUM;
proc sort data = &out.; by &time.;
run;

/* Censor extension */
%if "&censEXT." = "1" %then
  %km_censEXT(&out., &time., &StraN., &cHeight.,
      &cWidth., cens);
/* No. at risk */
%if "&atrisk." = "1" %then
  %km_atrisk(PLE, &stra., &time., &StraN.,
      &atriskorder., &Base., &Step., &Size., &label.,
      &afont., atrisk);
/* Homogeneity test */
%if "&Test." = "1" %then
  %km_homtest(Test, &TestX., &TestY., &Type.,
      &Size.,  &afont., HTEST);
/* Hazard ratio */
%if "&HR." = "1" %then
  %km_hr(HazardRatio, &HRX., &HRY., &StraN., &Step.,
      &Size.,  &afont., HR);
/* Confidence interval */
%if "&CI." = "1" %then
  %km_ci(&out., &time., &StraN., &Pattern., CI);
/* Quantile estimate */
%if "&MST." = "1" %then
  %km_qest(QEST, &stra., &MlabX., &MmedX., &MciX.,
      &MSTY., &StraN., &Step., &Size., &afont., QE);


%if %eval(&censEXT.+&atrisk.+&test.+&hr.+&ci.)
    ^= 0 %then %do;
data &anno.;
  length when $1. style $30.;
  set
    %if &CI. = 1 %then CI;
    %if &censEXT. = 1 %then cens;
    %if &atrisk. = 1 %then atrisk;
    %if &test. = 1 %then HTEST;
    %if &hr. = 1 %then HR;
    %if &MST. = 1 %then QE;
    ;
 retain when 'B';
run;
%end;

proc datasets lib = work;
  delete CS PLE QEST HazardRatio Test
    %if &CI. = 1 %then CI;
    %if &censEXT. = 1 %then cens;
    %if &atrisk. = 1 %then atrisk;
    %if &test. = 1 %then HTEST;
    %if &hr. = 1 %then HR;
    %if &MST. = 1 %then QE;
    ;
run; quit;
%mend km_data;



/********************************************************/
/* confidence interval */
%macro km_ci(
  ConfidenceInterval, time, StraN, pattern, out
);
%do i = 1 %to &StraN.;
data &out.&i.;
  length color function $10.;
  retain xsys ysys '2' style "&pattern." line 0 tmp .;
  set &ConfidenceInterval.(firstobs=2);
  where SL&i. ^= .;
  color = "&&scolor&i.";
  function = 'bar'; x = &time.; y = tmp; output;
  tmp = SU&i.;
  function = 'move'; x = &time.; y = SL&i.; output;
  keep xsys ysys style line color function x y;
%end;
data &out.;
set
%do i = 1 %to &StraN.;
&out.&i.(firstobs=2)
%end;;
%do i = 1 %to &StraN.;
proc datasets lib = work;
delete &out.&i.;
%end;;
run;
%mend km_ci;



/********************************************************/
/* censor extension */
%macro km_censEXT(
  data, time, StraN, cHeight, cWidth, out
);
data _null_;
  set &data.;
  %if "&StraN." = "1" %then
    call symput("exht", &cHeight.);
  %else
    call symput("exht", &cHeight. * (1 -
        min(of CSDF1 - CSDF&StraN.)) );;
data &out.;
  length color function $10.; 
  retain xsys ysys '2' size &cWidth.; set &data.;
  %do i = 1 %to &StraN.;
    if CSDF&i. ^= . then do;
      color = "&&color&i.";
      function='move'; x=&time.; y=CSDF&i.; output;
      function='draw'; x=&time.; y=CSDF&i. + &cHeight.; output;
    end;
  %end;
  keep xsys ysys size color function x y;
run;
%mend km_censEXT;



/********************************************************/
/* No. at risk */
%macro km_atrisk(
  data, stra, time, StraN, order,
  base, step, size, label, afont, out
);
data &out.;
  set &data.; by STRATUM &time.;
  do x = &order.;
    if &time. = 0 & Censor = . then do; leave; end;
    if &time. < x then do; leave; end;
  end;
data &out.;
  length color function $10.; set &out.; by STRATUM x;
  retain xsys '2' ysys '1' color 'black' position '3'
    function 'label' size &size.;
  %do i = &StraN. %to 1 %by -1;
    if STRATUM = &i. then y =
      (&base. + &step. * (&StraN. - 1)) - &step. * (&i. - 1);
  %end;
  text = cat(Left);
  if last.x then output;
data dummy;
  length color function $10.; 
  retain xsys '2' ysys '1' color 'black' position '3'
    function 'label' size &size.;
  do STRATUM = 1 to &StraN.;
  do x = &order.;
    %do i = &StraN. %to 1 %by -1;
      if STRATUM = &i. then y =
        (&base. + &step. * (&StraN. - 1)) - &step. * (&i. - 1);
    %end;
    output;
  end; end;
data &out.;
  merge dummy &out.; by STRATUM x;
  format xtext $20.; retain xtext '';
  if text ^= "" then xtext = text;
  else text = xtext;
data Label;
  length color function $10. text $100.;
  retain xsys '2' ysys '1' color 'black' position '3'
    function 'label' size &size.;
  x = 0; y = &base. + &step. * &StraN.; text = &label.; output;
data &out.; set &out. Label; retain style &afont.;
  keep xsys ysys color position function size x y text style;
proc datasets lib = work; delete dummy Label;
run;
%mend km_atrisk;



/********************************************************/
/* homogeneity test */
%macro km_homtest(
  Test, TestX, TestY, Type, Size, afont, out
);
data &out.;
  %if "&Type" = "logrank" %then set &Test.(firstobs=1 obs=1);
  %else %if "&Type" = "wilcoxon" %then
    set &Test.(firstobs=2 obs=2);
  %else %if "&Type" = "likelihoodratio" %then
    set &Test.(firstobs=3 obs=3);;
  retain xsys '1' ysys '1' color 'black' position '7'
    function 'label' size &size. style &afont.;
  x = &TestX.; y = &TestY.;
  %if "&Type" = "logrank" %then
    text = cat("Log-rank P = ", put(ProbChiSq, PVALUE5.4));
  %else %if "&Type" = "wilcoxon" %then
    text = cat("Wilcoxon P = ", put(ProbChiSq, PVALUE5.4));
  %else %if "&Type" = "likelihoodratio" %then
    text = cat("-2Log(LR) P = ", put(ProbChiSq, PVALUE5.4));;
  keep xsys ysys color position function size style x y text;
run;
%mend km_homtest;



/********************************************************/
/* hazard ratio */
%macro km_hr(
  HazardRatio, HRX, HRY, StraN, Step, Size, afont, out
);
data &out.;
  set &HazardRatio.;
  length text $200.;
  retain xsys '1' ysys '1' color 'black' position '7' function 'label'
      size &size. style &afont.;
  x = &HRX.;
  y = &HRY. - &Step. * (_n_ - 1);
  text = cat(
           compress(ClassVal0), " HR = ", 
           compress(put(HazardRatio, 8.3)), " [", 
           compress(put(HRLowerCL, 8.3)), ", ",
           compress(put(HRUpperCL, 8.3)), "]"
         );
  keep xsys ysys color position function size style x y text;
run;
%mend km_hr;



/********************************************************/
/* Quantile estimate + 95% CI */
%macro km_qest(
  Est, stra, MlabX, MmedX, MciX, MSTY, StraN, Step,
  Size, afont, out
);
data &out.;
  length text $200.;
  set &Est.;
  retain xsys '1' ysys '1' color 'black' function 'label'
      size &size. style &afont.;
  position = '4';
  x = &MmedX.; y = &MSTY. - &Step.; text = "MST"; output;
  x = &MciX.;  y = &MSTY. - &Step.; text = "95% C.I."; output;
  %do i = 1 %to &StraN.;
  if (STRATUM = &i. & Percent = 50) then do;
    position = '4';
    y = &MSTY. - &Step. * (&i. + 1);
    x = &MmedX.; text = put(Estimate, 6.1); output;
    x = &MciX.;
    text = cat("[", compress(put(LowerLimit, 6.1)),
                         ", ", compress(put(UpperLimit, 6.1)), ")"); output;
    position = '6';
    x = &MlabX.; text = trim(&stra.); output;
  end;
  %end;
  keep xsys ysys color position function size style x y text;
run;
%mend km_qest;

